#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/device.h>

#define TEMPLATE "template"
static dev_t template_dev_number;
static struct cdev *driver_object;
struct class *template_class;
static struct device *template_dev;

#ifdef CONFIG_PM /* Powermanagement */
static int template_suspend(struct device *dev )
{
	dev_info(dev,"template_runtime_suspend( %p )\n", dev );
	return 0;
}

static int template_resume(struct device *dev)
{
	dev_info(dev,"template_runtime_resume( %p )\n",dev);
	return 0;
}
#endif

static const struct dev_pm_ops template_class_pm = {
	.suspend = template_suspend,
	.resume  = template_resume,
};

static int __init template_init( void )
{
	if( alloc_chrdev_region(&template_dev_number,0,1,TEMPLATE)<0 )
		return -EIO;
	driver_object = cdev_alloc();
	if( driver_object==NULL )
		goto free_device_number;
	driver_object->owner = THIS_MODULE;
	if( cdev_add(driver_object,template_dev_number,1) )
		goto free_cdev;
	template_class = class_create( THIS_MODULE, TEMPLATE );
	if( IS_ERR( template_class ) ) {
		pr_err("template: no udev support\n");
		goto free_cdev;
	}
	template_dev = device_create( template_class, NULL,
		template_dev_number, NULL, "%s", TEMPLATE );
#ifdef CONFIG_PM /* Powermanagement */
	template_class->pm = &template_class_pm;
#endif
	return 0;
free_cdev:
	kobject_put( &driver_object->kobj );
free_device_number:
	unregister_chrdev_region( template_dev_number, 1 );
	return -EIO;
}

static void __exit template_exit( void )
{
	device_destroy( template_class, template_dev_number );
	class_destroy( template_class );
	cdev_del( driver_object );
	unregister_chrdev_region( template_dev_number, 1 );
	return;
}

module_init( template_init );
module_exit( template_exit );
MODULE_LICENSE( "GPL" );
